# encoding: utf-8

"""
    http utils
    ~~~~~~~~~~
    
    Small helpers around getting a webpage via http GET.
    
    You can easy get a web page encoding in unicode with HttpRequest().
    
    HttpRequest() and HTTPHandler2() make it possible to get the complete
    sended request headers. See also:
    http://stackoverflow.com/questions/603856/get-urllib2-request-headers
    
    examples:
    ~~~~~~~~~
    
    Get a page as unicode:
        from django_tools.utils.http import HttpRequest
        r = HttpRequest("http://www.google.com")
        print r.get_unicode()
        
    Get the request/response headers:
        from django_tools.utils.http import HttpRequest
        r = HttpRequest("http://www.google.com")
        response = r.get_response()
        print "Request headers as list:", response.request_headers
        print "Raw Request header:", response.request_header
    
    more info, see DocStrings below...
    
    :copyleft: 2011 by the django-tools team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

import cgi
import httplib
import re
import socket
import urllib2


class HTTPConnection2(httplib.HTTPConnection):
    """
    Like httplib.HTTPConnection but stores the request headers.
    Used in HTTPConnection3(), see below.
    """
    def __init__(self, *args, **kwargs):
        httplib.HTTPConnection.__init__(self, *args, **kwargs)
        self.request_headers = []
        self.request_header = ""

    def putheader(self, header, value):
        self.request_headers.append((header, value))
        httplib.HTTPConnection.putheader(self, header, value)

    def send(self, s):
        self.request_header = s
        httplib.HTTPConnection.send(self, s)

if hasattr(httplib, 'HTTPS'):
    class HTTPSConnection2(httplib.HTTPSConnection):
        """
        Like httplib.HTTPConnection but stores the request headers.
        Used in HTTPConnection3(), see below.
        """
        def __init__(self, *args, **kwargs):
            httplib.HTTPSConnection.__init__(self, *args, **kwargs)
            self.request_headers = []
            self.request_header = ""

        def putheader(self, header, value):
            self.request_headers.append((header, value))
            httplib.HTTPSConnection.putheader(self, header, value)

        def send(self, s):
            self.request_header = s
            httplib.HTTPSConnection.send(self, s)


class HTTPConnectionWrapper(object):
    """
    Wrapper around HTTPConnection2
    Used in HTTPHandler2(), see below.
    """
    def __init__(self, conn_class):
        self._conn_class = conn_class

    def __call__(self, *args, **kwargs):
        """
        instance made in urllib2.HTTPHandler.do_open()
        """
        self._conn = self._conn_class(*args, **kwargs)
        self.request_headers = self._conn.request_headers
        self.request_header = self._conn.request_header
        return self

    def __getattribute__(self, name):
        """
        Redirect attribute access to the local HTTPConnection() instance.
        """
        if name in ("_conn", "_conn_class"):
            return object.__getattribute__(self, name)
        else:
            return getattr(self._conn, name)


class HTTPHandler2(urllib2.HTTPHandler):
    """
    A HTTPHandler which stores the request headers.
    Used HTTPConnection3, see above.
    
    >>> opener = urllib2.build_opener(HTTPHandler2)
    >>> opener.addheaders = [("User-agent", "Python test")]
    >>> response = opener.open('http://www.python.org/')
   
    Get the request headers as a list build with HTTPConnection.putheader():
    >>> response.request_headers
    [('Accept-Encoding', 'identity'), ('Host', 'www.python.org'), ('Connection', 'close'), ('User-Agent', 'Python test')]
   
    >>> response.request_header.split("\\r\\n")[0]
    'GET / HTTP/1.1'
    """
    def http_open(self, req):
        conn_instance = HTTPConnectionWrapper(HTTPConnection2)
        response = self.do_open(conn_instance, req)
        response.request_headers = conn_instance.request_headers
        response.request_header = conn_instance.request_header
        return response

if hasattr(httplib, 'HTTPS'):
    class HTTPSHandler2(urllib2.HTTPSHandler):
        def https_open(self, req):
            conn_instance = HTTPConnectionWrapper(HTTPSConnection2)
            response = self.do_open(conn_instance, req)
            response.request_headers = conn_instance.request_headers
            response.request_header = conn_instance.request_header
            return response


class HttpRequest(object):
    """
    Helper class for easy request a web page and encode the response into unicode.
    Used HTTPHandler2, so the complete request headers are available.
    
    timeout
    ~~~~~~~
    HttpRequest() can take the argument 'timeout' but this works only since Python 2.6
    For Python < 2.6 the timeout TypeError would be silently catch.
    Activate a work-a-round with 'threadunsafe_workaround' to use socket.setdefaulttimeout()
    But this is not thread-safty!
    more info: 
        http://kurtmckee.livejournal.com/32616.html (Supporting a timeout in feedparser)
    
    examples
    ~~~~~~~~
    
    >>> r = HttpRequest("http://www.heise.de")
    >>> r.request.add_header("User-agent", "Python test")
    >>> response = r.get_response()
       
    List of all headers, used to create the Request:
    >>> response.request_headers
    [('Accept-Encoding', 'identity'), ('Host', 'www.heise.de'), ('Connection', 'close'), ('User-Agent', 'Python test')]
    
    The used Request as Text:
    >>> response.request_header.split("\\r\\n")[0]
    'GET / HTTP/1.1'
           
    
    Get the response httplib.HTTPMessage instance:
    
    >>> info = response.info()
    >>> info["content-type"]
    'text/html; charset=utf-8'

    >>> response.getcode()
    200
    >>> response.geturl()
    'http://www.heise.de'

    
    Get the content in unicode:
    
    >>> content = r.get_unicode()
    >>> isinstance(content, unicode)
    True
    >>> content[:14].lower()
    u'<!doctype html'
    
    
    If some encodings wrong, these list stored the tried encodings:
    
    >>> r.tried_encodings
    []
    
    
    Work's with https, too:
    >>> r = HttpRequest("https://encrypted.google.com")
    >>> r.request.add_header("User-agent", "Python https test")
    >>> content = r.get_unicode()
    >>> isinstance(content, unicode)
    True
    >>> content[:14].lower()
    u'<!doctype html'
    >>> response.request_header.split("\\r\\n")[0]
    'GET / HTTP/1.1'
    """
    _charset_re = None

    def __init__(self, url, timeout=None, threadunsafe_workaround=False):
        self.request = urllib2.Request(url=url)
        self.timeout = timeout
        self.threadunsafe_workaround = threadunsafe_workaround

        handlers = [HTTPHandler2]
        if hasattr(httplib, 'HTTPS'):
            handlers.append(HTTPSHandler2)

        self.opener = urllib2.build_opener(*handlers)

        # set in get_response()
        self.response_header = None
        self.response = None

        # filled in get_unicode()
        self.tried_encodings = []

    def get_response(self):
        """
        Cached access to response object.
        """
        if self.response is None:
            try:
                self.response = self.opener.open(self.request, timeout=self.timeout)
            except TypeError, err:
                # timeout argument is new since Python v2.6
                if not "timeout" in str(err):
                    raise

                if self.threadunsafe_workaround:
                    # set global socket timeout
                    old_timeout = socket.getdefaulttimeout()
                    socket.setdefaulttimeout(self.timeout)

                self.response = self.opener.open(self.request)

                if self.threadunsafe_workaround:
                    # restore global socket timeout
                    socket.setdefaulttimeout(old_timeout)

            self.response_header = self.response.info() # httplib.HTTPMessage instance
        return self.response

    def get_content(self):
        response = self.get_response()
        content = response.read()
        return content

    def get_content_type(self):
        content_type = self.response_header.get("content-type")
        content_type, params = cgi.parse_header(content_type)
        return content_type, params

    def get_encoding_from_content_type(self):
        content_type, params = self.get_content_type()
        if "charset" in params:
            return params["charset"].strip("'\"")

    def get_encodings_from_content(self, content):
        if self._charset_re is None:
            self._charset_re = re.compile(
                r'<meta.*?charset=["\']*(.+?)["\'>]', flags=re.I
            )
        return self._charset_re.findall(content)

    def get_unicode(self):
        """
        Returns the requested content back in unicode.
        Tried:
            1. charset from content-type
            2. every encodings from <meta ... charset=XXX>
            3. fall back and replace all unicode characters
        """
        content = self.get_content()

        # Try charset from content-type
        encoding = self.get_encoding_from_content_type()
        if encoding:
            try:
                return unicode(content, encoding)
            except UnicodeError:
                self.tried_encodings.append(encoding)

        # Try every encodings from <meta ... charset=XXX>
        encodings = self.get_encodings_from_content(content)
        for encoding in encodings:
            if encoding in self.tried_encodings:
                continue
            try:
                return unicode(content, encoding)
            except UnicodeError:
                self.tried_encodings.append(encoding)

        # Fall back:
        return unicode(content, encoding, errors="replace")


if __name__ == "__main__":
    print "Run doctests..."
    import doctest
    print doctest.testmod()
    print "-"*79

    r = HttpRequest("http://www.python.org/index.html", timeout=3, threadunsafe_workaround=True)
    response = r.get_response()
    print response.request_header
    print response.info()
    print repr(r.get_unicode())

    print "-"*79

    r = HttpRequest("https://raw.github.com/jedie/django-tools/master/README.creole", timeout=3, threadunsafe_workaround=True)
    r.request.add_header("User-agent", "Python test")
    r.request.add_header("Referer", "/")
    response = r.get_response()
    print response.request_header
    print response.info()
    print repr(r.get_unicode())



# coding: utf-8

"""
    some additional template filters
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    :copyleft: 2007-2009 by the django-tools team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

import datetime

from django_tools.utils.time_utils import datetime2float


if __name__ == "__main__":
    # For doctest only
    import os
    os.environ["DJANGO_SETTINGS_MODULE"] = "django.conf.global_settings"

from django.template.defaultfilters import stringfilter
from django.utils.translation import ugettext as _
from django.utils.encoding import force_unicode


CHMOD_TRANS_DATA = (
    u"---", u"--x", u"-w-", u"-wx", u"r--", u"r-x", u"rw-", u"rwx"
)
def chmod_symbol(octal_value):
    """
    Transform a os.stat().st_octal_value octal value to a symbolic string.
    ignores meta infromation like SUID, SGID or the Sticky-Bit.
    e.g. 40755 -> rwxr-xr-x
    >>> chmod_symbol(644)
    u'rw-r--r--'
    >>> chmod_symbol(40755)
    u'rwxr-xr-x'
    >>> chmod_symbol("777")
    u'rwxrwxrwx'
    """
    octal_value_string = str(octal_value)[-3:] # strip "meta info"
    return u''.join(CHMOD_TRANS_DATA[int(num)] for num in octal_value_string)
chmod_symbol.is_safe = True
chmod_symbol = stringfilter(chmod_symbol)


def get_oct(value):
    """
    Convert an integer number to an octal string.
    """
    try:
        return oct(value)
    except:
        return value
get_oct.is_safe = False


def human_duration(t):
    """
    Converts a time duration into a friendly text representation.
    
    >>> human_duration("type error")
    Traceback (most recent call last):
        ...
    TypeError: human_duration() argument must be timedelta, integer or float)
    
    
    >>> human_duration(datetime.timedelta(microseconds=1000))
    u'1.0 ms'
    >>> human_duration(0.01)
    u'10.0 ms'
    >>> human_duration(0.9)
    u'900.0 ms'
    >>> human_duration(datetime.timedelta(seconds=1))
    u'1.0 sec'
    >>> human_duration(65.5)
    u'1.1 min'
    >>> human_duration((60 * 60)-1)
    u'59.0 min'
    >>> human_duration(60*60)
    u'1.0 hours'
    >>> human_duration(1.05*60*60)
    u'1.1 hours'
    >>> human_duration(datetime.timedelta(hours=24))
    u'1.0 days'
    >>> human_duration(2.54 * 60 * 60 * 24 * 365)
    u'2.5 years'
    """
    if isinstance(t, datetime.timedelta):
        # timedelta.total_seconds() is new in Python 2.7
        t = datetime2float(t)
    elif not isinstance(t, (int, float)):
        raise TypeError("human_duration() argument must be timedelta, integer or float)")

    chunks = (
      (60 * 60 * 24 * 365, _('years')),
      (60 * 60 * 24 * 30, _('months')),
      (60 * 60 * 24 * 7, _('weeks')),
      (60 * 60 * 24, _('days')),
      (60 * 60, _('hours')),
    )

    if t < 1:
        return _("%.1f ms") % round(t * 1000, 1)
    if t < 60:
        return _("%.1f sec") % round(t, 1)
    if t < 60 * 60:
        return _("%.1f min") % round(t / 60, 1)

    for seconds, name in chunks:
        count = t / seconds
        if count >= 1:
            count = round(count, 1)
            break
    return "%(number).1f %(type)s" % {'number': count, 'type': name}
human_duration.is_safe = True


if __name__ == "__main__":
    import doctest
    print doctest.testmod(verbose=False)



# coding: utf-8

"""
    Dynamic SITE ID unittests
    ~~~~~~~~~~~~~~~~~~~~~~~~~
    
    :copyleft: 2012 by the django-tools team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""


from django_tools.dynamic_site.test_app.views import display_site
from django.conf.urls.defaults import patterns


urlpatterns = patterns('', (r'^display_site/$', display_site))



'''
Comparing two html documents.

from:
https://github.com/gregmuellegger/django/blob/soc2011%2Fform-rendering/django/test/html.py
'''
import re
import HTMLParser


WHITESPACE = re.compile('\s+')


def normalize_whitespace(string):
    return WHITESPACE.sub(' ', string)


class Element(object):
    def __init__(self, name, attributes):
        self.name = name
        self.attributes = sorted(attributes)
        self.children = []

    def append(self, element):
        if isinstance(element, basestring):
            element = normalize_whitespace(element)
            if self.children:
                if isinstance(self.children[-1], basestring):
                    self.children[-1] += element
                    self.children[-1] = normalize_whitespace(self.children[-1])
                    return
        elif self.children:
            # removing last children if it is only whitespace
            # this can result in incorrect dom representations since
            # whitespace between inline tags like <span> is significant
            if isinstance(self.children[-1], basestring):
                if self.children[-1].isspace():
                    self.children.pop()
        if element:
            self.children.append(element)

    def finalize(self):
        def rstrip_last_element(children):
            if children:
                if isinstance(children[-1], basestring):
                    children[-1] = children[-1].rstrip()
                    if not children[-1]:
                        children.pop()
                        children = rstrip_last_element(children)
            return children

        rstrip_last_element(self.children)
        for i, child in enumerate(self.children):
            if isinstance(child, basestring):
                self.children[i] = child.strip()
            elif hasattr(child, 'finalize'):
                child.finalize()

    def __eq__(self, element):
        if self.name != element.name:
            return False
        if len(self.attributes) != len(element.attributes):
            return False
        if self.attributes != element.attributes:
            # attributes without a value is same as attribute with value that
            # equals the attributes name:
            # <input checked> == <input checked="checked">
            for i in range(len(self.attributes)):
                attr, value = self.attributes[i]
                other_attr, other_value = element.attributes[i]
                if value is None:
                    value = attr
                if other_value is None:
                    other_value = other_attr
                if attr != other_attr or value != other_value:
                    return False
        if self.children != element.children:
            return False
        return True

    def __ne__(self, element):
        return not self.__eq__(element)

    def _count(self, element, count=True):
        if not isinstance(element, basestring):
            if self == element:
                return 1
        i = 0
        for child in self.children:
            # child is text content and element is also text content, then
            # make a simple "text" in "text"
            if isinstance(child, basestring):
                if isinstance(element, basestring):
                    if count:
                        i += child.count(element)
                    elif element in child:
                        return 1
            else:
                i += child._count(element, count=count)
                if not count and i:
                    return i
        return i

    def __contains__(self, element):
        return self._count(element, count=False) > 0

    def count(self, element):
        return self._count(element, count=True)

    def __getitem__(self, key):
        return self.children[key]

    def __unicode__(self):
        output = u'<%s' % self.name
        for key, value in self.attributes:
            if value:
                output += u' %s="%s"' % (key, value)
            else:
                output += u' %s' % key
        if self.children:
            output += u'>\n'
            output += u''.join(unicode(c) for c in self.children)
            output += u'\n</%s>' % self.name
        else:
            output += u' />'
        return output

    def __str__(self):
        return str(unicode(self))

    def __repr__(self):
        return unicode(self)


class RootElement(Element):
    def __init__(self):
        super(RootElement, self).__init__(None, ())

    def __unicode__(self):
        return u''.join(unicode(c) for c in self.children)


class HTMLParser2(HTMLParser.HTMLParser):
    """   
    Patched version of HTMLParser.HTMLParser with patch from:
        http://bugs.python.org/issue670664
    See also:
        https://github.com/gregmuellegger/django/issues/1
    """
    def __init__(self):
        HTMLParser.HTMLParser.__init__(self)
        self.cdata_tag = None

    def set_cdata_mode(self, tag):
        self.interesting = HTMLParser.interesting_cdata
        self.cdata_tag = tag.lower()

    def clear_cdata_mode(self):
        self.interesting = HTMLParser.interesting_normal
        self.cdata_tag = None

    # Internal -- handle starttag, return end or -1 if not terminated
    def parse_starttag(self, i):
        self.__starttag_text = None
        endpos = self.check_for_whole_start_tag(i)
        if endpos < 0:
            return endpos
        rawdata = self.rawdata
        self.__starttag_text = rawdata[i:endpos]

        # Now parse the data between i+1 and j into a tag and attrs
        attrs = []
        match = HTMLParser.tagfind.match(rawdata, i + 1)
        assert match, 'unexpected call to parse_starttag()'
        k = match.end()
        self.lasttag = tag = rawdata[i + 1:k].lower()

        while k < endpos:
            m = HTMLParser.attrfind.match(rawdata, k)
            if not m:
                break
            attrname, rest, attrvalue = m.group(1, 2, 3)
            if not rest:
                attrvalue = None
            elif attrvalue[:1] == '\'' == attrvalue[-1:] or \
                 attrvalue[:1] == '"' == attrvalue[-1:]:
                attrvalue = attrvalue[1:-1]
                attrvalue = self.unescape(attrvalue)
            attrs.append((attrname.lower(), attrvalue))
            k = m.end()

        end = rawdata[k:endpos].strip()
        if end not in (">", "/>"):
            lineno, offset = self.getpos()
            if "\n" in self.__starttag_text:
                lineno = lineno + self.__starttag_text.count("\n")
                offset = len(self.__starttag_text) \
                         - self.__starttag_text.rfind("\n")
            else:
                offset = offset + len(self.__starttag_text)
            self.error("junk characters in start tag: %r"
                       % (rawdata[k:endpos][:20],))
        if end.endswith('/>'):
            # XHTML-style empty tag: <span attr="value" />
            self.handle_startendtag(tag, attrs)
        else:
            self.handle_starttag(tag, attrs)
            if tag in self.CDATA_CONTENT_ELEMENTS:
                self.set_cdata_mode(tag) # <--------------------------- Changed
        return endpos

    # Internal -- parse endtag, return end or -1 if incomplete
    def parse_endtag(self, i):
        rawdata = self.rawdata
        assert rawdata[i:i + 2] == "</", "unexpected call to parse_endtag"
        match = HTMLParser.endendtag.search(rawdata, i + 1) # >
        if not match:
            return -1
        j = match.end()
        match = HTMLParser.endtagfind.match(rawdata, i) # </ + tag + >
        if not match:
            if self.cdata_tag is not None: # *** add ***
                self.handle_data(rawdata[i:j]) # *** add ***
                return j # *** add ***
            self.error("bad end tag: %r" % (rawdata[i:j],))
        # --- changed start ---------------------------------------------------
        tag = match.group(1).strip()
        if self.cdata_tag is not None:
            if tag.lower() != self.cdata_tag:
                self.handle_data(rawdata[i:j])
                return j
        # --- changed end -----------------------------------------------------
        self.handle_endtag(tag.lower())
        self.clear_cdata_mode()
        return j

class Parser(HTMLParser2):
    SELF_CLOSING_TAGS = ('br' , 'hr', 'input', 'img', 'meta', 'spacer',
        'link', 'frame', 'base', 'col')

    def __init__(self):
        HTMLParser2.__init__(self)
        self.root = RootElement()
        self.open_tags = []
        self.element_positions = {}

    def error(self, msg):
        raise HTMLParser.HTMLParseError(msg, self.getpos())

    def format_position(self, position=None, element=None):
        if not position and element:
            position = self.element_positions[element]
        if position is None:
            position = self.getpos()
        if hasattr(position, 'lineno'):
            position = position.lineno, position.offset
        return 'Line %d, Column %d' % position

    @property
    def current(self):
        if self.open_tags:
            return self.open_tags[-1]
        else:
            return self.root

    def handle_startendtag(self, tag, attrs):
        self.handle_starttag(tag, attrs)
        if tag not in self.SELF_CLOSING_TAGS:
            self.handle_endtag(tag)

    def handle_starttag(self, tag, attrs):
        element = Element(tag, attrs)
        self.current.append(element)
        if tag not in self.SELF_CLOSING_TAGS:
            self.open_tags.append(element)
        self.element_positions[element] = self.getpos()

    def handle_endtag(self, tag):
        if not self.open_tags:
            self.error("Unexpected end tag `%s` (%s)" % (
                tag, self.format_position()))
        element = self.open_tags.pop()
        while element.name != tag:
            if not self.open_tags:
                self.error("Unexpected end tag `%s` (%s)" % (
                    tag, self.format_position()))
            element = self.open_tags.pop()

    def handle_data(self, data):
        self.current.append(data)

    def handle_charref(self, name):
        self.current.append('&%s;' % name)

    def handle_entityref(self, name):
        self.current.append('&%s;' % name)


def parse_html(html):
    parser = Parser()
    parser.feed(html)
    parser.close()
    document = parser.root
    document.finalize()
    # Removing ROOT element if it's not necessary
    if len(document.children) == 1:
        if not isinstance(document.children[0], basestring):
            document = document.children[0]
    return document

if __name__ == "__main__":
    parse_html('<input value="Foo" type="text" name="first_name"></input>')



# coding: utf-8

"""
    Dynamic SITE ID unittests
    ~~~~~~~~~~~~~~~~~~~~~~~~~
    
    :copyleft: 2012 by the django-tools team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

import unittest

if __name__ == "__main__":
    # run unittest directly
    import os
    os.environ["DJANGO_SETTINGS_MODULE"] = "test_settings"

from django.contrib.sites.models import Site
from django.test import TestCase
from django.utils import log

from django_tools.unittest_utils.BrowserDebug import debug_response
from django_tools.dynamic_site.models import SiteAlias
from django_tools.unittest_utils.unittest_base import BaseTestCase


class FakeResponse(object):
    _charset = "utf-8"
    def __init__(self, content, status_code=200):
        self.content = content
        self.status_code = status_code


class DynamicSiteTest(BaseTestCase, TestCase):
    def setUp(self):
        Site.objects.all().delete()
        self.site1 = Site.objects.create(domain="domain_one.tld", name="site 1")
        self.site2 = Site.objects.create(domain="domain_two.tld", name="site 2")

        self.alias1 = SiteAlias.objects.create(site=self.site1, alias="domain_one_alias_one", regex=False)
        self.alias2 = SiteAlias.objects.create(site=self.site1, alias="domain_one_alias_two", regex=False)
        self.alias3 = SiteAlias.objects.create(site=self.site2, alias="www.domain_one.tld", regex=False)
        self.alias4 = SiteAlias.objects.create(site=self.site2, alias=r"^(.*?\.domain_two\.tld)$", regex=True)

    def test_fallback(self):
        response = self.client.get("/display_site/") # request with 'testserver'
        self.assertEqual(response.content,
            'ID from settings: 1 - id from get_current(): 1'
        )

    def test_no_alias1(self):
        response = self.client.get("/display_site/", HTTP_HOST="domain_one.tld")
        self.assertEqual(response.content,
            'ID from settings: 1 - id from get_current(): 1'
        )

    def test_no_alias2(self):
        response = self.client.get("/display_site/", HTTP_HOST="domain_two.tld")
        self.assertEqual(response.content,
            'ID from settings: 2 - id from get_current(): 2'
        )

    def test_string_alias1(self):
        response = self.client.get("/display_site/", HTTP_HOST="domain_one_alias_one")
        self.assertEqual(response.content,
            'ID from settings: 1 - id from get_current(): 1'
        )

    def test_string_alias2(self):
        response = self.client.get("/display_site/", HTTP_HOST="domain_one_alias_two")
        self.assertEqual(response.content,
            'ID from settings: 1 - id from get_current(): 1'
        )

    def test_regex_alias1(self):
        response = self.client.get("/display_site/", HTTP_HOST="foo.domain_two.tld")
        self.assertEqual(response.content,
            'ID from settings: 2 - id from get_current(): 2'
        )

    def test_regex_alias2(self):
        response = self.client.get("/display_site/", HTTP_HOST="foo.bar.domain_two.tld")
        self.assertEqual(response.content,
            'ID from settings: 2 - id from get_current(): 2'
        )

    def test_change(self):
        response = self.client.get("/display_site/", HTTP_HOST="www.domain_one.tld")
        self.assertEqual(response.content,
            'ID from settings: 2 - id from get_current(): 2'
        )

        self.alias3.site = self.site1
        self.alias3.save() # Should clean all site caches

        response = self.client.get("/display_site/", HTTP_HOST="www.domain_one.tld")
        self.assertEqual(response.content,
            'ID from settings: 1 - id from get_current(): 1'
        )

    def test_create(self):
        host = "not_exist_yet.tld"

        # Fallback to default:
        response = self.client.get("/display_site/", HTTP_HOST=host)
        self.assertEqual(response.content,
            'ID from settings: 1 - id from get_current(): 1'
        )

        # Should clean all site caches:
        SiteAlias.objects.create(site=self.site2, alias=host, regex=False)

        response = self.client.get("/display_site/", HTTP_HOST=host)
        self.assertEqual(response.content,
            'ID from settings: 2 - id from get_current(): 2'
        )






if __name__ == "__main__":
    # Run this unittest directly

    # Enable logging to console
    logger1 = log.getLogger("django_tools.DynamicSite")
    log.logging.basicConfig(format='%(created)f %(message)s')
    logger1.setLevel(log.logging.DEBUG)
    logger2 = log.getLogger("django_tools.local_sync_cache")
    logger2.setLevel(log.logging.DEBUG)

    from django.core import management
    management.call_command('test', 'dynamic_site')



# coding: utf-8

"""
    Dynamic SITE ID unittests
    ~~~~~~~~~~~~~~~~~~~~~~~~~
    
    :copyleft: 2012 by the django-tools team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from django.http import HttpResponse
from django.conf import settings
from django.contrib.sites.models import Site



def display_site(request):

    settings_id = settings.SITE_ID
    current_site = Site.objects.get_current()
    current_id = current_site.id

    txt = "ID from settings: %r - id from get_current(): %r" % (
        settings_id, current_id
    )
    return HttpResponse(txt)



